#!/usr/bin/env python

# Copyright (c) 2009 Patrick Dawson <pkdawson@gmail.com>
# Licensed under the terms of the GNU LGPL version 3:
#  http://www.gnu.org/licenses/lgpl.html

""" A lightweight plugin framework. """

from functools import partial
from inspect import isclass

import os
import imp
import logging

class Plugin(object):
    """
    name = 'Plugin Name'
    description = 'Short plugin description goes here'
    author = 'Nobody <nobody@example.com>'
    version = '1.0'
    
    def load(self, api):
        # hook into api, activate extensions
    def unload(self, api):
        # unhook from api, deactivate extensions
    - OR -
    def load(self):
        # activate extensions
    def unload(self):
        # deactivate extensions
    """


class PluginManager(object):
    """ Finds, loads and unloads plugins. """

    def __init__(self, api=None, plugin_type=Plugin, plugin_dirs=[]):
        if api: self.api = [api]
        else: self.api = []
        self.plugin_type = plugin_type
        self.plugins = {}
        self.plugin_dirs = set(plugin_dirs)
        self.found_plugins = []

    def loadPlugin(self, plugin):
        """ Instantiates and loads a plugin of the given class. """
        if plugin not in self.plugins:
            plug = plugin()
            plug.load(*self.api)
            self.plugins[plugin] = plug

    def unloadPlugin(self, plugin):
        """ Unload the plugin of the given class. """
        self.plugins[plugin].unload(*self.api)
        del self.plugins[plugin]
        
    def loadAll(self):
        """ Load all available plugins. """
        for plugin in self.found_plugins:
            self.loadPlugin(plugin)

    def unloadAll(self):
        """ Unload all plugins. """
        for plugin in list(self.plugins.keys()):
            self.unloadPlugin(plugin)
        assert self.plugins == {}

    def addPluginDirectory(self, pdir):
        """ Add a directory to be searched by findPlugins. """
        self.plugin_dirs.add(pdir)
        
    def removePluginDirectory(self, pdir):
        """ Remove a directory from the search path. """
        self.plugin_dirs.remove(pdir)
        
    def get(self, name):
        """ Get plugin by name. """
        for plugin in self.found_plugins:
            if plugin.name == name:
                return plugin
        return None

    def findPlugins(self):
        """ Search directories, import modules, and return a list of classes
        that inherit directly from the plugin_type given in the constructor. """
        found_plugins = []
        for pdir in self.plugin_dirs:
            to_import = set()
            for fn in os.listdir(pdir):
                fpath = os.path.join(pdir, fn)
                if os.path.isfile(fpath):
                    mod_name = os.path.splitext(fn)[0]
                else:
                    mod_name = fn
                to_import.add(mod_name)
            
            for mod_name in to_import:
                new_mod = None
                try:
                    fp, pathname, desc = imp.find_module(mod_name, [pdir])
                    try:
                        new_mod = imp.load_module(mod_name, fp, pathname, desc)
                    finally:
                        if fp: fp.close()
                except ImportError as err:
                    logging.debug(err)
                except SyntaxError as err:
                    logging.error(err)
                if new_mod:
                    mstuff = [getattr(new_mod, x) for x in dir(new_mod)]
                    mstuff = [x for x in mstuff if isclass(x)]
                    # each plugin must inherit directly from the plugin_type
                    new_plugins = [x for x in mstuff
                        if self.plugin_type in x.__bases__]
                    # each plugin requires at least a name
                    new_plugins = [x for x in new_plugins if hasattr(x, 'name')]
                    found_plugins.extend(new_plugins)
                    
        self.found_plugins = found_plugins
        return found_plugins



# optional stuff follows

class ExtensionPointMeta(type):
    """ Generic extension point, adapted from:
    http://www.djangosnippets.org/snippets/542/
    """
    def __init__(cls, name, bases, attrs):
        if cls.__name__ == 'ExtensionPoint':
            pass
        elif not hasattr(cls, 'extensions'):
            cls.extensions = []
        else:
            cls.activate = partial(
                lambda self: cls.extensions.append(cls), cls)
            cls.deactivate = partial(
                lambda self: cls.extensions.remove(cls), cls)

class AutoExtensionPointMeta(type):
    def __init__(cls, name, bases, attrs):
        if cls.__name__ == 'AutoExtensionPoint':
            pass
        elif not hasattr(cls, 'extensions'):
            cls.extensions = []
        else:
            cls.extensions.append(cls)

ExtensionPoint = ExtensionPointMeta('ExtensionPoint', (object,), {})
AutoExtensionPoint = AutoExtensionPointMeta('AutoExtensionPoint', (object,), {})
