#!/usr/bin/env python

import unittest
import sys; sys.path = ['..'] + sys.path
import slagroom

from iface import TestExtPoint, CheckLoadingAPI
from iface import CustomPluginType1, CustomPluginType2
from iface import AutoExtPoint

import aplug

class TestNoAPI(unittest.TestCase):
    def setUp(self):
        self.pm = slagroom.PluginManager()
        self.pm.addPluginDirectory('plugins')
        
    def testFindInDirectory(self):
        plugin_list = self.pm.findPlugins()
        self.assertEqual(len(plugin_list), 1)
        plug = plugin_list[0]
        self.assertEqual(plug.name, 'Test Plugin')
        
    def testGet(self):
        self.pm.findPlugins()
        self.assertEqual(self.pm.get('Test Plugin').name, 'Test Plugin')

class TestWithAPI(unittest.TestCase):
    def setUp(self):
        self.api = CheckLoadingAPI()
        self.pm = slagroom.PluginManager(api=self.api)
        self.pm.addPluginDirectory('plugins')

    def testLoad(self):
        plugin_list = self.pm.findPlugins()
        self.assertEqual(len(self.api.loaded), 0)
        self.pm.loadPlugin(plugin_list[0])
        self.assertEqual(len(self.api.loaded), 1)
        self.pm.unloadPlugin(plugin_list[0])
        self.assertEqual(len(self.api.loaded), 0)
        
    def testExtensions(self):
        plugin_list = self.pm.findPlugins()
        self.assertEqual(self._doSum(), 0)
        self.pm.loadPlugin(plugin_list[0])
        self.assertEqual(self._doSum(), 3)
        self.pm.unloadAll()
        self.assertEqual(self._doSum(), 0)
        
    def testLoadAll(self):
        self.pm.findPlugins()
        self.assertEqual(self._doSum(), 0)
        self.pm.loadAll()
        self.assertEqual(self._doSum(), 3)
        self.pm.unloadAll()
        self.assertEqual(self._doSum(), 0)
        
        
    def _doSum(self):
        mysum = 0
        for ext in TestExtPoint.extensions:
            mysum += ext.num()
        return mysum

class TestCustomPluginTypes(unittest.TestCase):
    def setUp(self):
        self.pm = slagroom.PluginManager(plugin_type=CustomPluginType1)
        self.pm.addPluginDirectory('plugins')
    
    def testFind(self):
        plugin_list = self.pm.findPlugins()
        self.assertEqual(len(plugin_list), 1)
        self.assertEqual(plugin_list[0].name, 'Cool Plugin 1')
        
    def testGet(self):
        self.pm.findPlugins()
        self.assertEqual(self.pm.get('Cool Plugin 1').name, 'Cool Plugin 1')
        
    def testGetNonexistent(self):
        self.pm.findPlugins()
        self.assertEqual(self.pm.get('kwyjibo'), None)
        
class TestMultipleDirs(unittest.TestCase):
    def setUp(self):
        self.pm = slagroom.PluginManager(plugin_type=CustomPluginType2)
        self.pm.addPluginDirectory('plugins')
        self.pm.addPluginDirectory('moreplugins')
    
    def testFind(self):
        plugin_list = self.pm.findPlugins()
        self.assertEqual(len(plugin_list), 2)
        self.assertNotEqual(self.pm.get('Cool Plugin 2'), None)
        self.assertNotEqual(self.pm.get('Another Plugin'), None)
        
    def testRemoveDir(self):
        plugin_list = self.pm.findPlugins()
        self.assertEqual(len(plugin_list), 2)
        self.pm.removePluginDirectory('plugins')
        plugin_list = self.pm.findPlugins()
        self.assertEqual(len(plugin_list), 1)
        self.assertEqual(self.pm.get('Cool Plugin 2'), None)
        self.assertNotEqual(self.pm.get('Another Plugin'), None)
        
class TestConstructorWithDir(unittest.TestCase):
    def testOneDir(self):
        pm = slagroom.PluginManager(plugin_type=CustomPluginType2,
            plugin_dirs=['plugins'])
        plugin_list = pm.findPlugins()
        self.assertEqual(len(plugin_list), 1)
        self.assertNotEqual(pm.get('Cool Plugin 2'), None)
    
    def testTwoDirs(self):
        pm = slagroom.PluginManager(plugin_type=CustomPluginType2,
            plugin_dirs=['plugins', 'moreplugins'])
        plugin_list = pm.findPlugins()
        self.assertEqual(len(plugin_list), 2)
        self.assertNotEqual(pm.get('Cool Plugin 2'), None)
        self.assertNotEqual(pm.get('Another Plugin'), None)
        

class TestAutoExtPoint(unittest.TestCase):
    def testAutoLoad(self):
        self.assertEqual(len(AutoExtPoint.extensions), 1)
        ext = AutoExtPoint.extensions[0]()
        self.assertEqual(ext.five(), 5)

if __name__ == '__main__':
    import logging
    logging.basicConfig(level=logging.CRITICAL)
    unittest.main()
